@@ -12,11 +12,12 @@ from requests.auth import HTTPBasicAuth
 import yaml
 import time
 
-VERSION = '0.4.2'
+VERSION = '0.5.0'
 
 backliftURL = {}
 default_backliftURL = 'https://www.backlift.com/'
-backlift_configfile = 'config.yml'
+backlift_appid_file = 'appid'
+backlift_cfg_folder = '.backlift'
 backlift_email = 'support@backlift.com'
 backlift_dir = '.backlift_env'
 
@@ -28,7 +29,8 @@ maxfiles = resource.getrlimit(resource.RLIMIT_NOFILE)[0] - 6
 
 ERR_ID =       "Ooops! Something's wrong. I couldn't obtain an app id. To\n"+\
                "use the push or watch commands, you need an existing\n"+\
-               "backlift app created with either the create or init commands."
+               "backlift app created with either the create or init commands.\n"+\
+               "If you've just upgraded backlift, run 'backlift init [name]'"
 
 ERR_BADID =    "Ooops! Something's wrong. I couldn't obtain an app id from\n"+\
                "the server. Please send an angry email to %s." % backlift_email
@@ -42,7 +44,7 @@ ERR_SERVER =   "Ooops! Our server is not responding. Hopefully this is\n"+\
 ERR_WRITE =    "Ooops! We couldn't create a file. Please check to ensure\n"+\
                "you have permission to write to %s."
 
-ERR_INIT =     "This app has already been initialized."
+ERR_INIT =     "This app has already been initialized. Use -f to force."
 
 ERR_500 =      "Ooops! Our server is having trouble. We're looking into\n"+\
                "it! While we're at it, please check to make sure you're\n"+\
@@ -93,7 +95,6 @@ def check_response(r):
 
 def listFiles(path, skip_hidden):
     files = []        
-    configfile = ''
 
     for (dirpath, dirnames, filenames) in os.walk(path):
         # import pdb; pdb.set_trace()
@@ -106,8 +107,9 @@ def listFiles(path, skip_hidden):
 
                 filepath = os.path.normpath(os.path.join(dirpath, name))
 
-                if name == backlift_configfile and dirpath == path:
-                    configfile = filepath
+                if dirpath == os.path.join(path, backlift_cfg_folder):
+                    continue
+
                 elif skip_hidden and name[0] == '.':
                     continue
 
@@ -117,7 +119,7 @@ def listFiles(path, skip_hidden):
     if filecount > maxfiles:
         fail(ERR_MAXFILE % (filecount, maxfiles))
 
-    return files, configfile
+    return files
 
 
 def collectFiles(path, skip_hidden):
@@ -125,7 +127,7 @@ def collectFiles(path, skip_hidden):
     rootpath = os.path.normpath(os.path.abspath(path))
     print "Scanning %s" % rootpath
 
-    filelist, configfile = listFiles(rootpath, skip_hidden)
+    filelist = listFiles(rootpath, skip_hidden)
 
     files = {}
 
@@ -137,36 +139,34 @@ def collectFiles(path, skip_hidden):
         except OSError:
             continue
 
-    return files, configfile
+    return files
 
 
 def scanFiles(path, skip_hidden):
     rootpath = os.path.normpath(os.path.abspath(path))
-    filelist, configfile = listFiles(rootpath, skip_hidden)
+    filelist = listFiles(rootpath, skip_hidden)
     times = {}
     for f in filelist:
         try:
             times[f] = os.stat(f).st_mtime
         except OSError:
             continue
-    return times, configfile
+    return times
 
 
-
-def get_id(configfile):
-    id = ''
+def get_id(path):
+    id_str = ''
+    id_file = os.path.join(path, backlift_cfg_folder, backlift_appid_file)
     try:
-        handle = open(configfile, 'r')
-        cfg_str = handle.read()
-        cfg_data = yaml.safe_load(cfg_str)
+        handle = open(id_file, 'r')
+        id_str = handle.read()
         handle.close()
-        id = cfg_data['_app_id']
     except Exception as e:
         pass
 
-    if not id: fail(ERR_ID)
+    if not id_str: fail(ERR_ID)
 
-    return id
+    return id_str
 
 
 def open_create(path, perm):
@@ -217,6 +217,17 @@ def read_api_key():
     return api_key
 
 
+def setup_app(path, app_id, force=False):
+    filepath = os.path.join(path, backlift_cfg_folder, backlift_appid_file)
+
+    if os.path.exists(filepath) and not force:
+        fail(ERR_INIT)
+
+    keyfile = open_create(filepath, 'w')
+    keyfile.write(app_id)
+    keyfile.close()
+
+
 ## REQUESTS -------------------------------------------------------------------
 
 def get_app_meta(api_key, app_id):
@@ -237,19 +248,25 @@ def get_app_meta(api_key, app_id):
     return None
 
 
-def create_app(api_key, name, template):
+def create_app(path, api_key, name, template, force=False):
     try:
         # import pdb; pdb.set_trace()
         headers = {'x-requested-with': 'BackliftCLI',
                    'x-cli-version': VERSION}
+
+        r = requests.get(backliftURL['checkname'] % name, 
+                          auth=HTTPBasicAuth('api', api_key), 
+                          verify=False, headers=headers)
+        check_response(r)
+        app_id = json.loads(r.text)['_id']
+        setup_app(path, app_id, force=force)
+
         r = requests.post(backliftURL['create'], 
                           data={'appname': name, 'template': template},
                           auth=HTTPBasicAuth('api', api_key), 
                           verify=False, headers=headers)
         check_response(r)
-
-        r = json.loads(r.text)
-        return r['_id']
+        return json.loads(r.text)['_id']
 
     except requests.exceptions.ConnectionError:
         fail(ERR_SERVER)
@@ -319,10 +336,14 @@ def download_template_files(path, template, api_key, params=None):
 
                 filepath = os.path.abspath(os.path.join(path, f))
 
+                if os.path.exists(filepath):
+                    print "Skipping %s (file exists)" % f
+                    continue
+
                 newfile = open_create(filepath, 'wb')
                 newfile.write(r.content)
                 newfile.close()
-                print "creating %s" % os.path.join(path, f)
+                print "Creating %s" % f
 
     except requests.exceptions.ConnectionError:
         fail(ERR_SERVER)
@@ -360,29 +381,21 @@ def setup(api_key):
     save_api_key(api_key)
 
 
-def create(path, name, template='init', mkpath=True):
+def create(path, name, template='init', mkpath=True, force=False):
 
     # import pdb; pdb.set_trace()
 
     api_key = read_api_key()
 
     app_root = os.path.normpath(os.path.abspath(path))
-
     if mkpath:
         app_root = os.path.normpath(os.path.join(app_root, name))
 
-    cfg_path = os.path.join(app_root, backlift_configfile)
-
-    if (os.path.exists(cfg_path)):
-        fail(ERR_INIT)
- 
-    id = create_app(api_key, name, template)
+    id = create_app(app_root, api_key, name, template, force=force)
     if id:
-        download_template_files(app_root, template, api_key, {
-            'app_id': id
-        })
-
-        print "A new app has been created in the %s folder." % name
+        if mkpath:
+            print "Downloading template into %s" % app_root
+        download_template_files(app_root, template, api_key)
         print "This app will be called %s." % id
 
     else:
@@ -405,9 +418,7 @@ def delete(path, name, force=False):
                     os.path.abspath(
                         os.path.join(path, name)))
 
-    cfg_path = os.path.join(app_root, backlift_configfile)
-
-    app_id = get_id(cfg_path)
+    app_id = get_id(app_root)
 
     if app_id:
         delete_app(api_key, app_id)
@@ -418,8 +429,8 @@ def delete(path, name, force=False):
         fail(ERR_BADID)
 
 
-def init(path, name):
-    create(path, name, mkpath=False)
+def init(path, name, force=False):
+    create(path, name, mkpath=False, force=force)
 
 
 def push(path, skip_hidden):
@@ -428,11 +439,11 @@ def push(path, skip_hidden):
 
     api_key = read_api_key()
 
+    id = get_id(path)
+
     # scan path for files to upload 
 
-    files, configfile = collectFiles(path, skip_hidden)
-
-    id = get_id(configfile)
+    files = collectFiles(path, skip_hidden)
 
     upload_files(id, files, api_key)
 
@@ -447,12 +458,12 @@ def watch(path, skip_hidden):
 
     id = push(path, skip_hidden)
 
-    before, configfile = scanFiles(path, skip_hidden)
+    before = scanFiles(path, skip_hidden)
 
     try:
         while 1:
             time.sleep (0.5)
-            after, configfile = scanFiles(path, skip_hidden)
+            after = scanFiles(path, skip_hidden)
             
             added = [f for f in after if not f in before]
             removed = [f for f in before if not f in after]
@@ -465,7 +476,7 @@ def watch(path, skip_hidden):
             
             if added or removed or modified:    
                 # import pdb; pdb.set_trace()
-                files, configfile = collectFiles(path, skip_hidden)        
+                files = collectFiles(path, skip_hidden)        
                 upload_files(id, files, api_key)
     
             before = after
@@ -480,10 +491,7 @@ def open_browser(path, admin=False):
 
     api_key = read_api_key()
 
-    cfg_path = os.path.normpath(os.path.abspath(
-        os.path.join(path, backlift_configfile)))
-
-    app_id = get_id(cfg_path)
+    app_id = get_id(path)
 
     meta = get_app_meta(api_key, app_id)
 
@@ -502,11 +510,8 @@ def make_su(path, username, password):
     # import pdb; pdb.set_trace()
 
     api_key = read_api_key()
-
-    cfg_path = os.path.normpath(os.path.abspath(
-        os.path.join(path, backlift_configfile)))
-
-    app_id = get_id(cfg_path)
+ 
+    app_id = get_id(path)
 
     userid = create_superuser(api_key, app_id, username, password)
 
@@ -538,7 +543,7 @@ def execute_command_line():
         help='The URL to backlift\'s server. Defaults to "%default"')
     parser.add_option('-f', '--force', 
         dest='force', action='store_const', const=True, default=False,
-        help='Force action. Used for delete only.')
+        help='Force action.')
     parser.add_option('-H', '--skip-hidden', 
         dest='skip_hidden', action='store_const', const=False, default=True,
         help='Toggle uploading of hidden files. (Files that start with a '+\
@@ -554,6 +559,7 @@ def execute_command_line():
     backliftURL['templatefiles'] = os.path.join(backlift_root, 'app-templates')
     backliftURL['upload'] = os.path.join(backlift_root, 'app-admin/%s/files/upload')
     backliftURL['createsuperuser'] = os.path.join(backlift_root, 'createsuperuser/%s')
+    backliftURL['checkname'] = os.path.join(backlift_root, 'checkname/%s')
 
     if len(args) < 1:
         parser.print_help()
@@ -574,9 +580,9 @@ def execute_command_line():
             else:
                 tokens = args[0].lower().split(':')
                 if len(tokens) > 1:
-                    create(options.path, args[1], tokens[1])
+                    create(options.path, args[1], tokens[1], force=options.force)
                 else:
-                    create(options.path, args[1], 'basic')
+                    create(options.path, args[1], 'basic', force=options.force)
         elif args[0].lower().startswith('delete'):
             if len(args) < 2:
                 appname = '.'
@@ -587,7 +593,7 @@ def execute_command_line():
             if len(args) != 2:
                 parser.print_help()
             else:
-                init(options.path, args[1])
+                init(options.path, args[1], force=options.force)
         elif args[0].lower() == 'open':
             open_browser(options.path)
         elif args[0].lower() == 'admin':
